local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

local HasteCDs = {
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
	26297--[[Berserking]],
	3045--[[Rapid Fire]],
	105919--[[Chronohunter]],
}

local function time_to_die()
	if s.MaxHealth("target") > 0 then
	-- Some very basic assumptions
		if s.InRaid() then
			if s.Boss("target") then
			-- Assume raid bosses to live a bout 5 minutes
				return s.HealthPercent("target") * 3
			else
				return s.HealthPercent("target") * 0.5
			end
		else
			if s.Boss("target") then
				return s.HealthPercent("target") * 1.2
			else
				return s.HealthPercent("target") * 0.1
			end
		end
	else
		return 31
	end
end

a.spells["Auto Shot"] = {
	ID = 75--[[Auto Shot]],
	EnemyTargetNeeded = 1,
	NotIfActive = true,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC and not s.Shooting()
	end,
	NoGCD = true,
}

a.spells["Pet Attack"] = {
	EnemyTargetNeeded = 1,
	Override = function(z)
		if x.NoCC and x.PetAlive and s.InCombat() and x.ActiveEnemy and not s.SameTargetAsPet("target") then
			s.FlashPet(PET_ACTION_ATTACK)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	EnemyTargetNeeded = 1,
	Override = function(z)
		if s.InCombat() and not x.PetActiveEnemy then
			s.FlashPet(PET_MODE_ASSIST)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Hunter's Mark"] = {
	ID = 1130--[[Hunter's Mark]],
	Debuff = 1130--[[Hunter's Mark]],
	EarlyRefresh = 5,
	CheckFirst = function(z)
		return not s.HasGlyph(53243--[[Glyph of Marked for Death]])
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aspect of the Hawk"] = {
	Type = "form",
	ID = 13165--[[Aspect of the Hawk]],
	Buff = 13165--[[Aspect of the Hawk]],
	BuffUnit = "player",
	NotWhileMoving = 1,
	CheckFirst = function(z)
		return not s.Moving("player")
	end,
}

a.spells["Aspect of the Fox while moving"] = {
	Type = "form",
	ID = 82661--[[Aspect of the Fox]],
	Buff = 82661--[[Aspect of the Fox]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return s.Moving("player") and not IsMounted()
	end,
}

a.spells["Explosive Trap (AoE)"] = {
	ID = 13813--[[Explosive Trap]],
	CheckFirst = function(z)
		return s.InCombat()
	end,
	FlashColor = "Purple",
	Continue = 1,
	NoRangeCheck = 1,
}

a.spells["Focus Fire at 5 stacks"] = {
	ID = 82692--[[Focus Fire]],
	RequireBuff = 19623--[[Frenzy]],
	RequireStack = 5,
	RequireBuffUnit = "pet",
	CheckFirst = function(z)
		return s.InCombat()
	end,
	NoGCD = 1,
}

a.spells["Serpent Sting if not ticking"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]])
	end,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Serpent Sting if not ticking and target HP <= 90%"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Serpent Sting if not ticking and TTD  >= 10"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]])
	end,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	Run = function(z)
		if time_to_die() >= 10 then
			z.Continue = 0
			z.FlashColor = "White"
		else
			z.Continue = 1
			z.FlashColor = "Pink"
		end
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Chimera Shot if target HP <= 90%"] = {
	ID = 53209--[[Chimera Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Bestial Wrath if focus > 60"] = {
	ID = 19574--[[Bestial Wrath]],
	CheckFirst = function(z)
		return s.InCombat() and s.Power("player", SPELL_POWER_FOCUS) > 60
	end,
}

a.spells["Multi-Shot (AoE)"] = {
	ID = 2643--[[Multi-Shot]],
	CheckFirst = function(z)
		return s.InCombat()
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Steady Shot (AoE)"] = {
	ID = 56641--[[Steady Shot]],
	CheckFirst = function(z)
		local _, time = s.LastSpellTravelTime(2643--[[Multi-Shot]], "target")
		if not time then
			return false
		else
			return s.InCombat() and x.NoCC and (GetTime() - time ) < 2
		end
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Cobra Shot (AoE)"] = {
	ID = 77767--[[Cobra Shot]],
	CheckFirst = function(z)
		local _, time = s.LastSpellTravelTime(2643--[[Multi-Shot]], "target")
		if not time then
			return false
		else
			return s.InCombat() and x.NoCC and (GetTime() - time ) < 2
		end
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = 1,
	Continue = 1,
}

a.spells["Kill Shot"] = {
	ID = 53351--[[Kill Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["A Murder of Crows"] = {
	ID = 120657--[[A Murder of Crows]],
	CheckFirst = function(z)
		return s.InCombat() and not (s.BuffDuration(HasteCDs, "player") > 0) 
	end,
	EnemyTargetNeeded = 1,
	NoRangeCheck = 1,
}

a.spells["A Murder of Crows with The Beast Within"] = {
	ID = 120657--[[A Murder of Crows]],
	RequireBuff = 34692--[[The Beast Within]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat()
	end,
	EnemyTargetNeeded = 1,
	NoRangeCheck = 1,
}

a.spells["Kill Command"] = {
	ID = 34026--[[Kill Command]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot with Thrill of the Hunt"] = {
	ID = 3044--[[Arcane Shot]],
	RequireBuff = 34720--[[Thrill of the Hunt]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot as focus dump when not hasted"] = {
	ID = 3044--[[Arcane Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and ((s.Power("player", SPELL_POWER_FOCUS) >= 66) or (s.SpellCooldown(53209--[[Chimera Shot]]) >= 4)) and not s.Buff(HasteCDs, "player")
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot if focus >= 69 or The Beast Within up"] = {
	ID = 3044--[[Arcane Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and ((s.Power("player", SPELL_POWER_FOCUS) >= 69) or s.Buff(34692--[[The Beast Within]],"player"))
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Arcane Shot if focus >= 67"] = {
	ID = 3044--[[Arcane Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.Power("player", SPELL_POWER_FOCUS) >= 67
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Steady Shot with Steady Focus < 3"] = {
	ID = 56641--[[Steady Shot]],
	CheckFirst = function(z)
		local duration = s.MyBuffDuration(53224--[[Steady Focus]], "player")
		return s.InCombat() and x.NoCC and duration > 0 and duration < 3
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Steady Shot"] = {
	ID = 56641--[[Steady Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Cobra Shot"] = {
	ID = 77767--[[Cobra Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Rapid Fire without Haste buffs and without The Beast Within"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and not s.Buff(HasteCDs, "player") and not s.Buff(34692--[[The Beast Within]], "player")
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire without Haste buffs"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and not s.Buff(HasteCDs, "player")
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire without Haste buffs or TTD <= 30"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and (not s.Buff(HasteCDs, "player") or (time_to_die() <= 30))
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Readiness after Rapid Fire"] = {
	ID = 23989--[[Readiness]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.SpellCooldown(3045--[[Rapid Fire]]) > 0
	end,
	Continue = 1,
	NoGCD = 1,
	FlashColor = "Yellow",
}

a.spells["Aimed Shot with Master Marksman"] = {
	ID = 19434--[[Aimed Shot]],
	RequireMyBuff = 82926--[[Fire!]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aimed Shot over 90% HP"] = {
	ID = 19434--[[Aimed Shot]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") > 90
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Aimed Shot as focus dump when hasted"] = {
	ID = 19434--[[Aimed Shot]],
	RequireBuff = HasteCDs,
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and ((s.SpellCooldown(53209--[[Chimera Shot]]) > 5) or (s.Power("player", SPELL_POWER_FOCUS) >= 80))
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Call Pet"] = {
	ID = 883--[[Call Pet 1]],
	CheckFirst = function(z)
		return not x.PetAlive and not IsMounted()
	end,
	FlashID = {883--[[Call Pet 1]], 83242--[[Call Pet 2]], 83243--[[Call Pet 3]], 83244--[[Call Pet 4]], 83245--[[Call Pet 5]], 982--[[Revive Pet]]},
}

a.spells["Mend Pet"] = {
	ID = 136--[[Mend Pet]],
	CheckFirst = function(z)
		return x.PetAlive and s.Health("pet") > 0 and s.HealthPercent("pet") < 80
	end,
}

a.spells["Explosive Shot"] = {
	ID = 53301--[[Explosive Shot]],
	MyDebuffID = 53301--[[Explosive Shot]],
	EarlyRefresh = 1.9,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and not s.SpellOrAuraDelay(53301--[[Explosive Shot]], "target", true)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Black Arrow"] = {
	ID = 3674--[[Black Arrow]],
	MyDebuffID = 3674--[[Black Arrow]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	Run = function(z)
		if time_to_die() >= 8 then
			z.Continue = 0
			z.FlashColor = "White"
		else
			z.Continue = 1
			z.FlashColor = "Pink"
		end
	end,
	EnemyTargetNeeded = 1,
}